@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,12 +35,13 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-433279-02.js';
+var gTestfile = 'regress-452498-077.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 433279;
-var summary = 'Do not assert: pn != tc->parseContext->nodeList';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
+var actual = '';
+var expect = '';
+
 
 //-----------------------------------------------------------------------------
 test();
@@ -51,14 +52,14 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  try
-  {
-    eval('var {a} = b; c(d + 1);');
-  }
-  catch(ex)
-  {
-  }
+
+  expect = '1';
+
+// ------- Comment #77 From Brendan Eich
+
+  (function () { const [d] = [1]; [d] = [2]; print(actual = d);})();
+
+  actual = String(actual);
 
   reportCompare(expect, actual, summary);
 
