@@ -7,3 +7,5 @@ cp $1/src/*.c src/
 cp $1/AUTHORS ./AUTHORS
 patch -p4 <pause-resume.patch
 patch -p3 <bug495794_closeAudio.patch
+patch -p4 <sydney_os2_base.patch
+patch -p4 <sydney_os2_moz.patch
